/**
* Creates an instance of MaskEngine
*
* @class
* @name MaskEngine
* @param {ZhUtil} ZhUtil - An instance of ZhUtil.
*/
function MaskEngine(ZhUtil) {

	var FORMATER = "Format";
	var self = this;
	var PLAIN_TEXT_PASTE_TYPE = "text/plain";

	self.hasFormater = function(name) {
		return !!name && !!self[name + FORMATER];

	};

	self.getFormater = function(name) {
		if (!self.hasFormater(name)) {
			throw new Error("Formater '" + name + "' not found!");
		}
		return self[name + FORMATER];
	};

	function getCurrentLang(){
		var lang = ZhUtil.getLang() || '';
		var parts = lang.split('_');
		var defaultLang = 'en';
		var acceptedLangs = {
			en: 'en_us',
			pt: 'pt',
			pt_br: 'pt_br',
			es: 'es',
			es_co: 'es_co'
		};
		return acceptedLangs[lang] || acceptedLangs[parts[0]] || acceptedLangs[defaultLang];
	}

	function getDefaultOptions(params){
		var currentLang = getCurrentLang();
		return Util.extend($.currency.i18n[currentLang], (params || {}), true);
	}

	self.numberFormat = function(number, params) {
		var defaultOptions = getDefaultOptions(params);
		defaultOptions.symbol = "";
		defaultOptions.showSymbol = false;
		defaultOptions.precision = 0;
		defaultOptions.thousands = "";
		return self.currencyFormat(number, defaultOptions);
	};

	/**
     * Apply the percent mask to the given value
     *
     * @function percentFormat
     * @private
     * @param {object} value - Where the mask will be applied
     * @param {object} params - Params to apply the mask
     * @return {string} - Value with the mask applied
     */

	self.percentFormat = function(value, params) {
		if(Util.isString(value)){
			if(value[value.length - 1] === "%"){
				value = value.slice(0, value.length - 1);
			}
		}
		var defaultParams = {
			precision: 2,
			minValue: 0,
			maxValue: 100
		};
		params = Util.extend(defaultParams, params);
		var percentualValue = (value - params.minValue) / (params.maxValue - params.minValue) * 100;
		return percentualValue.formatFloat(params.precision) + "%";
	};

	self.floatFormat = function(value, params) {
		var defaultOptions = getDefaultOptions(params);
		defaultOptions.symbol = "";
		defaultOptions.showSymbol = false;
		return self.currencyFormat(value, defaultOptions);
	};

	self.currencyFormat = function(value, params) {
		var defaultOptions = getDefaultOptions(params);

		var valueFormated = "";
		try {
			var options = Util.extend(defaultOptions, (params || {}), true);
			valueFormated = $.currency.string(value, options);
		} catch (e) {
			valueFormated = value;
		}
		return valueFormated;
	};

	self.currency = function(element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;
		$.currency.setMask(params, element);
	};

	self.float = function(element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;

		params.symbol = '';
		params.showSymbol = false;
		$.currency.setMask(params, element);
	};

	self.date = function(element, viewValue, params, callBack) {
		$.mask.definitions.d = '[0-3]';
		$.mask.definitions.m = '[0-1]';
		var value = viewValue || '';
		element.val(value);
		element.mask("d9/m9/9999", {
			completed: function() {
				var value = this.val();
				callBack(value);
			}
		});
	};

	self.datetime = function(element, viewValue, params, callBack) {
		var DEFAULT_PARAM = "d9/m9/9999";
		var format = params.format || DEFAULT_PARAM;
		$.mask.definitions.d = '[0-3]';
		$.mask.definitions.m = '[0-1]';
		$.mask.definitions.h = '[0-2]';
		$.mask.definitions.s = '[0-5]';
		var value = viewValue;
		element.val(value);
		element.mask(format, {
			completed: function() {
				var value = this.val();
				callBack(value);
			}
		});
	};

	self.dateFormat = function(value, params) {
		params = params || {};
		var regSplit = new RegExp('\/|:|-', 'g');

		if (value && value.split(regSplit).length === 1) {
			value = parseInt(value);
		}

		return moment(value, params.inputFormat).format(params.format || "L LTS");
	};

	self.fix = function(element, viewValue, params, callBack) {
		if (element && params && params.mask) {
			var value = viewValue || '';
			element.val(value);

			params.mask = ZhUtil.evaluate(params.mask);
			if(angular.isFunction(params.mask))
				params.mask = params.mask();

			element.mask(params.mask, {
				completed: function() {
					var value = this.val();
					callBack(value);
				},
				autoclear: params.autoclear !== false
			});

			return element.bind('keyup paste blur', function(event) {
					if (event.originalEvent) {
						handlePasteOnFixMask(event.originalEvent, element, params);
					}
					if (element.mask().length === 0) {
						callBack(element.mask());
					}
			});
		}
	};

	function handlePasteOnFixMask(event, element, params) {
		if (event.clipboardData && event.clipboardData.types && event.clipboardData.getData) {
			if (~event.clipboardData.types.indexOf(PLAIN_TEXT_PASTE_TYPE)) {
				pastedData = event.clipboardData.getData(PLAIN_TEXT_PASTE_TYPE);

				var maskAcceptedValues = getCleanFixMaskPattern(params.mask);
				var clearedPastedData = Util.clearValue(pastedData, { type: "fix", params: params }, self);

				if (maskAcceptedValues.length === Util.clearValue(clearedPastedData.length)) {
					element.val(pastedData);
				}
			}
		}
	}

	function getCleanFixMaskPattern(pattern) {
		var fixMaskDefinitions = 'a9*';
		return pattern.replace(new RegExp("[^" + fixMaskDefinitions + "]", "g"), "");
	}

	self.fixMaskParser = function(value, params) {
		var parsedValue = "";
		var maskPattern = ZhUtil.evaluate(params.mask);
		var patternsTranslation = Util.getFixMaskPatternsTranslation();

		if (value && maskPattern) {
			var j = 0;
			for (var i = 0; i < maskPattern.length; i++) {
				var currentDigit = value.charAt(i);
				var patternDigit = maskPattern[j];
				var patternTranslation = patternsTranslation[patternDigit];
				if (patternTranslation && patternTranslation.pattern.test(currentDigit)) {
					parsedValue += currentDigit;
				} else {
					if(patternDigit){
						parsedValue += patternDigit;
					}
					if (!patternTranslation && currentDigit !== patternDigit) {
						parsedValue += currentDigit;
						j++;
					}
				}
				j++;
			}
		} else {
			parsedValue = value;
		}

		return parsedValue;
	};

	self.fixParser = function fixParser(value, params) {
		var parsedValue = "";
		var maskPattern = ZhUtil.evaluate(params.mask);
		var patternsTranslation = Util.getFixMaskPatternsTranslation();

		if (value && maskPattern) {
			for (var i = 0; i < maskPattern.length; i++) {
				var currentDigit = value.charAt(i);
				var patternDigit = maskPattern[i];
				var patternTranslation = patternsTranslation[patternDigit];
				if (patternTranslation && patternTranslation.pattern.test(currentDigit)) {
					parsedValue += currentDigit;
				} else if (!patternTranslation && currentDigit !== patternDigit) {
					parsedValue += currentDigit;
				}
			}
		} else {
			parsedValue = value;
		}

		return parsedValue;
	};

	self.number = function(element, viewValue, params) {
		element.on("keyup", function() {
			element.val(self.numberFormat(element.val(), params));
		});

		element.val(self.numberFormat(viewValue, params));
	};

	/**
     * Apply the percent mask to the given element
     *
     * @function percent
     * @private
     * @param {object} element - Where the mask will be applied
     * @param {string} viewValue - Value of the field passed by the Angular
     * @param {object} params - Params to apply the mask
     */

	self.percent = function(element, viewValue, params) {
		element.on("blur", function() {
			element.val(self.percentFormat(element.val(), params));
		});
	};

	var getLocaleSeparator = function() {
		var num = 1.1;
		return num.toLocaleString().slice(1, -1);
	};

	/**
     * Build a RegExp to zerofill mask with the current language format
     *
     * @function _zerofillRegExpBuilder
     * @private
     * @return {object} regex - RegExp to apply in the mask
     */
	var _zerofillRegExpBuilder = function(){
		var language = $.currency.i18n[window.lang];
		var decimal = language.decimal;
		var thousands = language.thousands;
		var regex = new RegExp("[^0-9" + thousands + decimal + "]", "g");
		return regex;
	};

	var _zerofill = function(value, length, params) {
		if (value) {
			value = value.toString();
			if (params.acceptOnlyNumbers) {
				value = value.replace(_zerofillRegExpBuilder(), '');
			}
			while (value.length < length) {
				value = "0" + value;
			}
		}
		return value;
	};

	var _zerofillRight = function(value, length, params) {
		if (Util.isEmptyOrBlank(value)) {
			value = "0";
		}

		value = value.toString();
		if (params.acceptOnlyNumbers) {
			value = value.replace(/[^0-9]/g, '');
		}
		var decimalSeparator = value.indexOf(',') !== -1 ? ',' : (value.indexOf('.') !== -1 ? '.' : getLocaleSeparator());
		value = value.split(decimalSeparator);

		if (Util.isEmptyOrBlank(value[0])) {
			value[0] = "0";
		}

		if (value.length === 1) {
			value[1] = "";
		}

		if (value[1].length > length) {
			value[1] = value[1].slice(0, length);
		}
		while (value[1].length < length) {
			value[1] += "0";
		}

		value.splice(2);
		return value.join(decimalSeparator);
	};

	var _getZerofillLength = function(element, params) {
		if (params && params.maxlength) {
			return params.maxlength || 2;
		} else {
			return element ? element.attr('maxlength') : 4;
		}
	};

	var _removeLetters = function(e) {
		if(e.keyCode > 64 && e.keyCode < 91){
			return false;
		} else {
			return true;
		}
	};

	self.zerofillFormat = function(value, params, element) {
		params = params || {};
		var length = _getZerofillLength(element, params);
		if (params.right) {
			value = _zerofillRight(value, length, params);
		} else {
			value = _zerofill(value, length, params);
		}
		return value;
	};

	self.zerofill = function(element, viewValue, params, callBack) {
		if (viewValue) {
			var value = self.zerofillFormat(viewValue, params, element);
			element.val(value);
			callBack(value);
		}

		element.on('keydown', function(e){
			if(params.acceptOnlyNumbers){
				return _removeLetters(e);
			} else {
				return true;
			}
		});

		return element.bind('change', function() {
			var value = self.zerofillFormat($(this).val(), params, $(this));
			element.val(value);
			callBack(value);
		});
	};

	/**
     * Apply the metric mask to the given element
     *
     * @function metric
     * @private
     * @param {object} element - Where the mask will be applied
     * @param {string} viewValue - Value of the field passed by the Angular
     * @param {object} params - Params to apply the mask
     */
	self.metric = function(element, viewValue, params) {
		element.on("blur", function() {
			element.val(self.metricFormat(element.val(), params));
		});

		element.val(self.metricFormat(viewValue, params));
	};


	var metricSuffixes = ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'];

	/**
     * Apply the metric mask to the given value
     *
     * @function metricFormat
     * @private
     * @param	{string} value - Value where the mask will be applied
     * @param	{object} params - Params to apply the mask
     * @return	{string} valueToReturn - Value with the mask applied
     */
	self.metricFormat = (function() {

		function metricFormat(value, params) {
			var valueToReturn = "";
			params = params || {};

			if (!Util.isEmptyOrBlank(value)) {
				if(Util.isString(value)){
					value = cleanValue(value, params);
					if(isNaN(value)){
						var suffix = getSuffix(value);
						value = parseFloat(value);
						value = metricDecode(value, suffix);
					} else {
						value = parseFloat(value);
					}
				}
				valueToReturn = dirtyValue(metricEncode(value, params.precision), params.decimal);
			}

			return valueToReturn;
		}

		/**
	     * Define and add the metric suffix to the value
	     *
	     * @function  metricEncode
	     * @private
	     * @param	{number} value - Value used to define the suffix
	     * @param	{number} precision - Used to round the value
	     * @return	{string} value with the metric mask
	     */
		function metricEncode(value, precision) {
			for (var i = 0; Math.abs(value) >= 1000; i++) {
				value /= 1000;
			}
			value = Util.roundDecimal(value, Util.validateOptionalValue(precision,2));
			return value + metricSuffixes[i];
		}

		/**
	     * Calculate the value without the suffix
	     *
	     * @function  metricDecode
	     * @private
	     * @param	{number} value - Value that will be calculated
	     * @param	{string} suffix - Suffix removed from the value
	     * @return	{number} value - Value without the suffix
	     */
		function metricDecode(value, suffix) {
			for (var i = 0; metricSuffixes[i] != suffix; i++) {
				value *= 1000;
			}
			return value;
		}

		/**
	     * Transform the value into a float
	     *
	     * @function cleanValue
	     * @private
	     * @param	{string} value - Value to transform
	     * @param	{object} params - Parameters to use on the conversion
	     * @return	{number} value - Value converted
	     */
		function cleanValue(value, params) {
			var thousandsRegexStr = "["+Util.validateOptionalValue(params.thousands, ".")+ "]";
			var thousandsRegex = new RegExp(thousandsRegexStr, "g");
			value = value.replace(thousandsRegex, '');
			value = value.replace(Util.validateOptionalValue(params.decimal, ","), '.');
			return value;
		}

		/**
	     * Transform the value into a String with the decimal signal
	     *
	     * @function dirtyValue
	     * @private
	     * @param	{string} value - Value to transform
	     * @param	{object} decimal - decimal signal
	     * @return	{number} value - Value converted
	     */
		function dirtyValue(value, decimal) {
			value = String(value);
			value = value.replace('.',Util.validateOptionalValue(decimal, ","));
			return value;
		}

		/**
	     * Extract and return the sufix from the value
	     *
	     * @function getSuffix
	     * @private
	     * @param	{string} value - Where the suffix will be extracted
	     * @return	{string} suffix - The extracted suffix
	     */
		function getSuffix(value) {
			var suffix = value.substr(-1);
			suffix = suffix === "k" ? suffix : suffix.toUpperCase();
			return isMetricSuffix(suffix) ? suffix : "";
		}

	    /**
	     * @function  isMetricSuffix
	     * @private
	     * @param	{string} suffix - given suffix
	     * @return	{boolean} verification result
	     */
		function isMetricSuffix(suffix) {
			return metricSuffixes.indexOf(suffix) !== -1;
		}

		return metricFormat;

	})();


	this.timeFormatToSeconds = function(value, withSeconds) {
		var seconds = 0;
		var splittedValue = value.split("H");

		if(splittedValue.length > 1) {
			seconds += splittedValue[0] * 3600;
			splittedValue.splice(0,1);
		}

		splittedValue = splittedValue[0].toString();
		splittedValue = splittedValue.split("m");

		if(splittedValue.length > 1) {
			seconds += splittedValue[0] * 60;
			splittedValue.splice(0,1);
		}

		if (withSeconds) {
			splittedValue = splittedValue[0].toString();
			splittedValue = splittedValue.split("s");
			seconds += Number(splittedValue[0]);
		}

		return seconds;
	};

	this.secondsToTimeFormat = function(value, params) {
		params = params ? params : {};

		value = parseFloat(value);
		var response = "";
		var withSeconds = params.withSeconds || false;

		if(value && value > 0) {
			var hours = parseInt(value / 3600);
			var minutes = parseInt((value % 3600 ) / 60);
			var seconds = parseInt((value % 3600) % 60);

			if(hours > 0) {
				response += hours + "H";
			}

			if(minutes > 0) {
				response += minutes + "m";
			}

			if(withSeconds && seconds > 0) {
				response += seconds + "s";
			}
		}

		return response;
	};

	this.getMappedMasks = function() {
		return maskNameToMaskFunction;
	};

	this.hasMask = function(maskName) {
		return maskNameToMaskFunction.hasOwnProperty(maskName);
	};

	this.registerMask = function(maskName, maskFunction, maskFormatFunction) {
		if (maskName && maskFunction) {
			if (!maskNameToMaskFunction[maskName]) {
				maskNameToMaskFunction[maskName] = {};
				maskNameToMaskFunction[maskName].mask = maskFunction;
				maskNameToMaskFunction[maskName].format = maskFormatFunction;
			} else {
				throw ScreenService.i18n("Mask __maskName__ is already mapped!", {
					maskName: maskName
				});
			}
		} else {
			throw ScreenService.i18n("Invalid parameters");
		}
	};

	var maskNameToMaskFunction = {
		"currency": {
			"mask": self.currency,
			"format": self.currencyFormat
		},
		"float": {
			"mask": self.float,
			"format": self.floatFormat
		},
		"date": {
			"mask": self.date,
			"format": self.dateFormat
		},
		"datetime": {
			"mask": self.datetime,
			"format": undefined
		},
		"fix": {
			"mask": self.fix,
			"format": self.fixMaskParser,
			"parser": self.fixParser
		},
		"number": {
			"mask": self.number,
			"format": self.numberFormat
		},
		"zerofill": {
			"mask": self.zerofill,
			"format": self.zerofillFormat
		},
		"metric": {
			"mask": self.metric,
			"format": self.metricFormat
		},
		"percent": {
			"mask": self.percent,
			"format": self.percentFormat
		},
		"time": {
			"mask": undefined,
			"format": self.secondsToTimeFormat
		}
	};
}